#! /usr/bin/env python

from  xml.dom.minidom import parse
import sys, getopt

debugprint = 0
HOSTPORT   = '50000'          # xinetd Port No.
NEUNET_EVENT_BSIZ = '8'       # event byte size of NEUNET
MAX_FILE_BSIZE_IN_MB = '1024' # Max. Mbyte size of event data file
MON_GNUPLOT_PATH = '/home/daq/gnuplot/bin/gnuplot' #Abs. path of gnuplot
MON_PNG_OUT_DIR  = '/home/daq/Data/png' #Directory of png histogram files
MON_ANAPANEL_TEMPLATE_FILE = MON_PNG_OUT_DIR + '/anapanel.tmpl' #Template file for anapanel html
#MON_TOF_MIN      = '0'                  #Min. value of TOF histograms
#MON_TOF_MAX      = '30000'              #Max. value of TOF histograms
#MON_TOF_BIN      = '500'                #Num. of bin of TOF histograms
NUM_PSD_PER_MOD  = '8'                  #Num. of PSDs in a NEUNET
#NUM_PIX_PER_PSD  = '500'                #Num. of pixel per a PSD
#MON_UPDATE_RATE  = '500'                #update rate of Monitor Comp.
GATENET_DEFAULT_ADDR = '192.168.0.15'

def read_simple_list(fname):
    nvList = [ ]

    for line in open(fname, 'r'):
        itemList= line[:-1].split('\t')
        nvList.append(itemList)
    return nvList

def make_root_begin():
    text =  '<?xml version="1.0"?>\n'
    text += '<?xml-stylesheet href="config.xsl" type="text/xsl"?>\n'
    text += '<configInfo>\n'
    return text

def make_root_end():
    text = '</configInfo>\n'
    return text

def make_daqoperator(addr):
    text =  '   <daqOperator>\n'
    text += '      <hostAddr>' + addr + '</hostAddr>\n'
    text += '   </daqOperator>\n'
    return text

def make_daq_groups_begin():
    if debugprint:
        print '   <daqGroups>'
    text = '   <daqGroups>\n'
    return text

def make_daq_groups_end():
    text = '   </daqGroups>\n'
    return text

def make_daq_group_begin(id):
    text= '      <daqGroup gid='+'"group'+ str(id) + '">\n'
    return text

def make_daq_group_end():
    text = '      </daqGroup>\n'
    return text

def make_daq_components_begin():
    text = '         <components>\n'
    return text

def make_daq_components_end():
    text = '         </components>\n'
    return text

def make_component_begin(name):
    text =  '            <component cid="'+ name + '0' + '">\n'
    return text

def make_component_inst(name):    
    text =  '               <instName>' + name + '0' + '.rtc' + '</instName>\n'
    text += '               <execPath>/home/daq/DaqComponents/bin/' + name + 'Comp</execPath>\n'
    text += '               <confFile>/home/daq/DaqComponents/rtc.conf</confFile>\n'
    return text

def make_component_end():
    text = '            </component>\n'
    return text

def make_host_addr(addr, port):
    if debugprint:    
        print '               <hostAddr>' + addr + '</hostAddr>'
    text  = '               <hostAddr>' + addr + '</hostAddr>\n'
    text += '               <hostPort>' + port + '</hostPort>\n'    
    return text

def make_start_order(order):
    text = '               <startOrd>' + str(order) + '</startOrd>\n'
    return text

def make_params_begin():
    text = '               <params>\n'
    return text

def make_params_end():
    text = '               </params>\n'
    return text
    
def make_param(attr, val):
    text = '                  <param pid="' + attr + '">' + val + '</param>\n'
    return text

def make_gatherer_port():
    text =  '               <inPorts>\n'
    text += '               </inPorts>\n'
    text += '               <outPorts>\n'
    text += '                  <outPort>gatherer_out</outPort>\n'
    text += '               </outPorts>\n'
    return text


def make_dispatcher_port():
    text =  '               <inPorts>\n'
    text += '                  <inPort from="Gatherer0:gatherer_out">dispatcher_in</inPort>\n'
    text += '               </inPorts>\n'
    text += '               <outPorts>\n'
    text += '                  <outPort>dispatcher_out1</outPort>\n'
    text += '                  <outPort>dispatcher_out2</outPort>\n'
    text += '               </outPorts>\n'
    return text

def make_logger_port():
    text = '               <inPorts>\n                  <inPort from="Dispatcher0:dispatcher_out1">logger_in</inPort>\n               </inPorts>\n               <outPorts>\n               </outPorts>\n'
    return text

def make_monitor_port():
    text =  '               <inPorts>\n'
    text += '                  <inPort from="Dispatcher0:dispatcher_out2">monitor_in</inPort>\n'
    text += '               </inPorts>\n'
    text += '               <outPorts>\n'
    text += '               </outPorts>\n'
    return text

def make_no_port():
    text =  '               <inPorts>\n'
    text += '               </inPorts>\n'
    text += '               <outPorts>\n'
    text += '               </outPorts>\n'
    return text    

def make_gatherer(addr, order, neunets, daqId):
    text = ''
    text += make_component_begin('Gatherer')
    text += make_host_addr(addr, HOSTPORT)
    text += make_component_inst('Gatherer')    
    text += make_start_order(order)
    text += make_gatherer_port()
    text += make_params_begin()
    text += make_param('daqId',str(daqId))

    i = 0
    neus = ''
    while i < len(neunets):
        neu = make_param('srcAddr', neunets[i])
        neus = neus + neu
        i+= 1
    text += neus
    text += make_params_end()
    text += make_component_end()
    return text

def make_dispatcher(addr, order, event, sampling):
    text = ''
    text += make_component_begin('Dispatcher')
    text += make_host_addr(addr, HOSTPORT)
    text += make_component_inst('Dispatcher')    
    text += make_start_order(order)
    text += make_dispatcher_port()
    text += make_params_begin()
    text += make_param('eventByteSize', NEUNET_EVENT_BSIZ)
    #text += make_param('monRate', '100')
    text += make_param('samplingRate', 'dispatcher_out2/' + str(sampling))
    text += make_params_end()   
    text += make_component_end()
    return text

def make_logger(addr, order, neunets, daqId, event, loggingDir, instId):
    text = ''
    text += make_component_begin('Logger')
    text += make_host_addr(addr, HOSTPORT)
    text += make_component_inst('Logger')    
    text += make_start_order(order)
    text += make_logger_port()
    text += make_params_begin()
    text += make_param('daqId',str(daqId))
    i = 0
    #while i < len(neunets[daqId]):
    neus = ''
    while i < len(neunets):        
        neu = make_param('srcAddr', neunets[i])
        neus = neus + neu
        i+= 1
    text += neus

    text += make_param('instId', instId)
    text += make_param('dirName', loggingDir)
    text += make_param('eventByteSize', NEUNET_EVENT_BSIZ)
    text += make_param('isLogging', 'yes')        
    text += make_param('maxFileSizeInMegaByte', MAX_FILE_BSIZE_IN_MB)
    text += make_params_end()   
    text += make_component_end()
    return text

def make_monitor(addr, order, neunets, daqId, sampling):
    text = ''
    text += make_component_begin('Monitor')
    text += make_host_addr(addr, HOSTPORT)
    text += make_component_inst('Monitor')    
    text += make_start_order(order)
    text += make_monitor_port()
    text += make_params_begin()
    text += make_param('daqId',str(daqId))
    i = 0

    neus = ''
    while i < len(neunets):        
        neu = make_param('srcAddr', neunets[i])
        neus = neus + neu
        i+= 1
    text += neus

    text += make_param('samplingRate', 'dispatcher_out2/' + str(sampling))
    #text += make_param('monRate', '100')
    text += make_param('gnuplot_path', MON_GNUPLOT_PATH)
    text += make_param('png_output_dir', MON_PNG_OUT_DIR)
    text += make_param('anapanel_template_file', MON_ANAPANEL_TEMPLATE_FILE)
    #text += make_param('tof_min', MON_TOF_MIN)
    #text += make_param('tof_max', MON_TOF_MAX)
    #text += make_param('num_of_tof_bin', MON_TOF_BIN)
    text += make_param('num_of_psd_per_module', NUM_PSD_PER_MOD)
    #text += make_param('num_of_pixel_per_psd',  NUM_PIX_PER_PSD)
    #text += make_param('monitor_update_rate',   MON_UPDATE_RATE) 
    text += make_params_end()
    text += make_component_end()     
    return text

def make_gatenet(addr, order):
    text = ''
    text += make_component_begin('Gatenet')
    text += make_host_addr(addr, HOSTPORT)
    text += make_component_inst('Gatenet')    
    text += make_start_order(order)
    text += make_no_port()
    text += make_params_begin()
    text += make_param('gatenetAddr',   GATENET_DEFAULT_ADDR) 
    text += make_params_end()    
    text += make_component_end()
    return text

def getGroupNum(nvList):
    groupNo = 0
    for item in nvList:
        if item[0] == '***':   
            groupNo = groupNo + 1
    return groupNo

def neunetList(groupNum, addr):
    lastoct = []
    neunets = []
    daqGroupNo = -1
    index = 0

    myaddr = addr.split(',')

    ser = []
    for i in myaddr:
        dash = False
        for j in i:
            st = 0
            sp = 0
            if j== '-':
                ser = i.split('-')
                st = int( ser[0] )
                sp = int( ser[1] ) + 1
                dash = True
                for k in range(st, sp):
                    lastoct.append( str(k) )
        if dash == False:
            l = str(i).strip()
            lastoct.append(l)

    for i in lastoct:
        addr = '192.168.0.' + i
        neunets.append(addr)
    #print neunets
    return neunets

